/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.information.service.impl;

import me.zhengjie.modules.dm.information.domain.DmInformation;
import me.zhengjie.utils.ValidationUtil;
import me.zhengjie.utils.FileUtil;
import lombok.RequiredArgsConstructor;
import me.zhengjie.modules.dm.information.repository.DmInformationRepository;
import me.zhengjie.modules.dm.information.service.DmInformationService;
import me.zhengjie.modules.dm.information.service.dto.DmInformationDto;
import me.zhengjie.modules.dm.information.service.dto.DmInformationQueryCriteria;
import me.zhengjie.modules.dm.information.service.mapstruct.DmInformationMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import me.zhengjie.utils.PageUtil;
import me.zhengjie.utils.QueryHelp;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.LinkedHashMap;

/**
* @website https://el-admin.vip
* @description 服务实现
* @author xwx
* @date 2022-06-06
**/
@Service
@RequiredArgsConstructor
public class DmInformationServiceImpl implements DmInformationService {

    private final DmInformationRepository dmInformationRepository;
    private final DmInformationMapper dmInformationMapper;

    @Override
    public Map<String,Object> queryAll(DmInformationQueryCriteria criteria, Pageable pageable){
        Page<DmInformation> page = dmInformationRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder),pageable);
        return PageUtil.toPage(page.map(dmInformationMapper::toDto));
    }

    @Override
    public List<DmInformationDto> queryAll(DmInformationQueryCriteria criteria){
        return dmInformationMapper.toDto(dmInformationRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder)));
    }

    @Override
    @Transactional
    public DmInformationDto findById(Long id) {
        DmInformation dmInformation = dmInformationRepository.findById(id).orElseGet(DmInformation::new);
        ValidationUtil.isNull(dmInformation.getId(),"DmInformation","id",id);
        return dmInformationMapper.toDto(dmInformation);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DmInformationDto create(DmInformation resources) {
        Snowflake snowflake = IdUtil.createSnowflake(1, 1);
        resources.setId(snowflake.nextId()); 
        return dmInformationMapper.toDto(dmInformationRepository.save(resources));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DmInformation resources) {
        DmInformation dmInformation = dmInformationRepository.findById(resources.getId()).orElseGet(DmInformation::new);
        ValidationUtil.isNull( dmInformation.getId(),"DmInformation","id",resources.getId());
        dmInformation.copy(resources);
        dmInformationRepository.save(dmInformation);
    }

    @Override
    public void deleteAll(Long[] ids) {
        for (Long id : ids) {
            dmInformationRepository.deleteById(id);
        }
    }

    @Override
    public void download(List<DmInformationDto> all, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (DmInformationDto dmInformation : all) {
            Map<String,Object> map = new LinkedHashMap<>();
            map.put("发布内容(需发布的信息内容)", dmInformation.getContent());
            map.put("(00-待审核  01-审批通过(已发布） 02-审核未通过)", dmInformation.getStatus());
            map.put("审批人", dmInformation.getApprovalUser());
            map.put("任务ID act_ru_task", dmInformation.getTaskId());
            map.put("任务实例id act_ru_execution", dmInformation.getProcInstId());
            map.put("发起人", dmInformation.getRequestUser());
            map.put("创建者", dmInformation.getCreateBy());
            map.put("更新者", dmInformation.getUpdateBy());
            map.put("创建日期", dmInformation.getCreateTime());
            map.put("更新时间", dmInformation.getUpdateTime());
            map.put("发布时间", dmInformation.getReleaseTime());
            map.put("审批时间", dmInformation.getApprovalTime());
            map.put("发布标题", dmInformation.getTitle());
            map.put("审批备注", dmInformation.getApprovalRemark());
            map.put("是否设置审批人 1-true 0-false", dmInformation.getIsHasApproval());
            map.put("是否有效", dmInformation.getIsValid());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }
}